@@ -1,11 +1,12 @@
 /*
-*  tcal.c:  Start `gcal' with date set ONE day ahead (default mode, like
-*             the `--shift=1' long-style option is passed to it).
+*  tcal.c:  Start `gcal' with date set ONE day ahead (default mode,
+*             like the `--shift=1' long-style option is passed to it).
 *             Useful if you call this program in `~/.profile' and you
-*             want to see all fixed dates warnings related to tomorrow + 1 day
+*             want to see all fixed dates related to tomorrow + 1 day
 *             (advanced  -ct  respectively  --period-of-fixed-dates=t  option).
+*             Works *only* for Gregorian years!
 *
-*  Copyright (C) 1996 Thomas Esken
+*  Copyright (C) 1995, 1996, 1997 Thomas Esken
 *
 *  This software doesn't claim completeness, correctness or usability.
 *  On principle I will not be liable for ANY damages or losses (implicit
@@ -28,14 +29,14 @@
 
 
 
-static char rcsid[]="$Id: tcal.c 2.40 1996/07/16 02:04:00 tom Exp $";
+static char rcsid[]="$Id: tcal.c 2.92 1997/03/31 02:09:02 tom Exp $";
 
 
 
 /*
 *  Include header files.
 */
-#include "gcal_tai.h"
+#include "tailor.h"
 #if HAVE_CTYPE_H
 #  include <ctype.h>
 #endif
@@ -48,10 +49,10 @@ static char rcsid[]="$Id: tcal.c 2.40 1996/07/16 02:04:00 tom Exp $";
 #if HAVE_ERRNO_H
 #  include <errno.h>
 #endif
+#if (!HAVE_SIGNAL_H || !HAVE_SIGNAL) && HAVE_SYS_TYPES_H   /* Otherwise "gcal.h" includes <sys/types.h> */
+#  include <sys/types.h>
+#endif
 #if HAVE_SYS_STAT_H
-#  if HAVE_SYS_TYPES_H
-#    include <sys/types.h>
-#  endif
 #  include <sys/stat.h>
 #endif
 #if TIME_WITH_SYS_TIME
@@ -75,6 +76,7 @@ static char rcsid[]="$Id: tcal.c 2.40 1996/07/16 02:04:00 tom Exp $";
 *   Program specific preprocessor symbols.
 */
 #define  ENV_VAR_GCALPROG  "GCALPROG"
+#define  SHIFT_VALUE_DFT   "1"
 
 
 
@@ -86,12 +88,13 @@ extern "C"
 {
 #endif
 LOCAL void
-usage_msg __P_((FILE *fp,
-                char *prgr_name,
-                int   exit_status));
+usage_msg __P_((      FILE *fp,
+                const char *prgr_name,
+                      int   exit_status));
 LOCAL void
-version_msg __P_((FILE *fp,
-                  int   exit_status));
+version_msg __P_((      FILE *fp,
+                  const char *prgr_name,
+                        int   exit_status));
 LOCAL VOID_PTR
 my_malloc __P_((const int   amount,
                 const int   exit_status,
@@ -113,6 +116,16 @@ my_error __P_((const int   exit_status,
                const long  module_line,
                const char *var_name,
                const int   var_contents));
+#if HAVE_SIGNAL && (defined(SIGINT) || defined(SIGTERM) || defined(SIGHUP))
+LOCAL RETSIGTYPE
+handle_signal __P_((int the_signal));
+#endif
+#if !HAVE_STRNCASECMP
+LOCAL int
+my_strncasecmp __P_((const char *s1,
+                     const char *s2,
+                           int   len));
+#endif /* !HAVE_STRNCASECMP */
 LOCAL int
 days_of_february __P_((const int year));
 LOCAL Bool
@@ -164,38 +177,70 @@ LOCAL char  *s1;                      /* General purpose text buffer */
 
    LOCAL void
 usage_msg (fp, prgr_name, exit_status)
-   FILE *fp;
-   char *prgr_name;
-   int   exit_status;
+         FILE *fp;
+   const char *prgr_name;
+         int   exit_status;
 /*
    Writes the program "usage" text to file `fp' and
      terminates the program with `exit_status'.
 */
 {
-#if USE_GER
+#if USE_DE
    fprintf(fp, "Aufruf:  %s  [--help | --version] | [--shift=[+|-]ZAHL] [ARGUMENT...]\n", prgr_name);
-#else /* !USE_GER */
+   if (!exit_status)
+    {
+      S_NEWLINE(fp);
+      fprintf(fp, "Fehlerberichte via eMail an <esken@uni-muenster.de>");
+      S_NEWLINE(fp);
+      fprintf(fp, "oder (falls das fehlschl%sgt) an <bug-gnu-utils@prep.ai.mit.edu>.", AE);
+      S_NEWLINE(fp);
+    }
+#else /* !USE_DE */
    fprintf(fp, _("Usage:  %s  [--help | --version] | [--shift=[+|-]NUMBER] [ARGUMENT...]\n"), prgr_name);
-#endif /* !USE_GER */
+   if (!exit_status)
+    {
+      S_NEWLINE(fp);
+      fprintf(fp, _("Email bug reports to <esken@uni-muenster.de>"));
+      S_NEWLINE(fp);
+      fprintf(fp, _("or (if this fails) to <bug-gnu-utils@prep.ai.mit.edu>."));
+      S_NEWLINE(fp);
+    }
+#endif /* !USE_DE */
    exit(exit_status);
 }
 
 
 
    LOCAL void
-version_msg (fp, exit_status)
-   FILE *fp;
-   int   exit_status;
+version_msg (fp, prgr_name, exit_status)
+         FILE *fp;
+   const char *prgr_name;
+         int   exit_status;
 /*
    Writes the program "version" text to file `fp' and
      terminates the program with `exit_status'.
 */
 {
-   char *ptr_rcsid=rcsid+5;
+   auto char  *ptr_rcsid=rcsid+12;
 
 
-   rcsid[36] = '\0';
-   fprintf(fp, "%s\n", ptr_rcsid);
+   rcsid[16] = '\0';
+   fprintf(fp, "%s (GNU cal %s) %s\n", prgr_name, VERSION_NO, ptr_rcsid);
+   fprintf(fp, "Copyright (C) 1995-1997 Thomas Esken\n");
+#if USE_DE
+   fprintf(fp, "Dies ist freie Software; in den Quellen befindet sich die Lizenz-");
+   S_NEWLINE(fp);
+   fprintf(fp, "und Kopierbedingung.  Es gibt KEINERLEI Garantie, nicht einmal f%sr", UE);
+   S_NEWLINE(fp);
+   fprintf(fp, "die TAUGLICHKEIT oder die VERWENDBARKEIT ZU EINEM ANGEGEBENEN ZWECK.");
+#else /* !USE_DE */
+   fprintf(fp, _("This is free software; see the source for copying conditions."));
+   S_NEWLINE(fp);
+   fprintf(fp, _("There is NO warranty, without even the implied warranty of"));
+   S_NEWLINE(fp);
+   fprintf(fp, _("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."));
+#endif /* !USE_DE */
+   S_NEWLINE(fp);
    exit(exit_status);
 }
 
@@ -230,7 +275,7 @@ my_malloc (amount, exit_status, module_name, module_line, var_name, var_contents
      */
      my_error (exit_status, module_name, module_line, var_name, var_contents);
 
-   return ptr_memblock;
+   return(ptr_memblock);
 }
 
 
@@ -257,7 +302,7 @@ my_realloc (ptr_memblock, amount, exit_status, module_name, module_line, var_nam
      */
      my_error (107, module_name, module_line, var_name, (int)testval);
    if (ptr_memblock == (VOID_PTR)NULL)
-     return my_malloc (amount, exit_status, module_name, module_line, var_name, var_contents);
+     return(my_malloc (amount, exit_status, module_name, module_line, var_name, var_contents));
    ptr_memblock = (VOID_PTR)realloc(ptr_memblock, (int)amount);
    if (ptr_memblock == (VOID_PTR)NULL)
      /*
@@ -265,7 +310,7 @@ my_realloc (ptr_memblock, amount, exit_status, module_name, module_line, var_nam
      */
      my_error (exit_status, module_name, module_line, var_name, var_contents);
 
-   return ptr_memblock;
+   return(ptr_memblock);
 }
 
 
@@ -282,29 +327,29 @@ my_error (exit_status, module_name, module_line, var_name, var_contents)
      and terminates the program with status `exit_status'.
 */
 {
-#if USE_GER
+#if USE_DE
    fprintf(stderr, "\n%s: Abbruch, ", prgr_name);
-#else /* !USE_GER */
+#else /* !USE_DE */
    fprintf(stderr, _("\n%s: abort, "), prgr_name);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
    switch(exit_status)
     {
-#if USE_GER
+#if USE_DE
       case 124:
-        fprintf(stderr, "`%s' Zeile %ld: virtueller Speicher ersch"OE"pft (%s=%d)",
-                module_name, module_line, var_name, var_contents);
+        fprintf(stderr, "`%s' Zeile %ld: virtueller Speicher ersch%spft (%s=%d)",
+                module_name, module_line, OE, var_name, var_contents);
         break;
       case 107:
-        fprintf(stderr, "`%s' Zeile %ld: (`%s') ung"UE"ltiger Wert f"UE"r Tabellengr"OE""SZ"e `sizeof %s>%d'",
-                module_name, module_line, INTERNAL_TXT, var_name, var_contents);
+        fprintf(stderr, "`%s' Zeile %ld: (`%s') ung%sltiger Wert f%sr Tabellengr%s%se `sizeof %s>%d'",
+                module_name, module_line, INTERNAL_TXT, UE, UE, OE, SZ, var_name, var_contents);
         break;
       case 2:
-        fprintf(stderr, "Versatzwert `%s' ist ung"UE"ltig", var_name);
+        fprintf(stderr, "Versatzwert `%s' ist ung%sltig", var_name, UE);
         break;
       default:
         fprintf(stderr, "`%s' Zeile %ld: (`%s') unbehandelter Fehler (%d)",
                 module_name, module_line, INTERNAL_TXT, exit_status);
-#else /* !USE_GER */
+#else /* !USE_DE */
       case 124:
         fprintf(stderr, _("`%s' line %ld: virtual memory exhausted (%s=%d)"),
                 module_name, module_line, var_name, var_contents);
@@ -319,13 +364,71 @@ my_error (exit_status, module_name, module_line, var_name, var_contents)
       default:
         fprintf(stderr, _("`%s' line %ld: (`%s') unmanaged error (%d)"),
                 module_name, module_line, _("Internal"), exit_status);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
     }
    S_NEWLINE(stderr);
    exit(exit_status);
 }
 
 
+
+#if HAVE_SIGNAL && (defined(SIGINT) || defined(SIGTERM) || defined(SIGHUP))
+   LOCAL RETSIGTYPE
+handle_signal (the_signal)
+   int the_signal;
+/*
+   Signal handler function which displays the numeric ID of the received signal
+     on stderr channel and terminates the program with an exit status 3.
+*/
+{
+   fflush(stdout);
+#if USE_DE
+   fprintf(stderr, "\n%s: Programmabbruch durch Signal %d\n", prgr_name, the_signal);
+#else /* !USE_DE */
+   fprintf(stderr, _("\n%s: program aborted by signal %d\n"), prgr_name, the_signal);
+#endif /* */  
+   exit(3);
+}
+#endif /* HAVE_SIGNAL && (SIGINT || SIGTERM || SIGHUP) */
+
+
+
+#if !HAVE_STRNCASECMP
+   PUBLIC int
+my_strncasecmp (s1, s2, len)
+   const char *s1;
+   const char *s2;
+         int   len;
+/*
+   Same as the ANSI C `strncmp()' function, but case insensitive.
+*/
+{
+   auto const Uchar  *p1=(const Uchar *)s1;
+   auto const Uchar  *p2=(const Uchar *)s2;
+   auto       Uchar   c1;
+   auto       Uchar   c2;
+
+
+   if (   !len
+       || p1 == p2)
+     return(0);
+   if (len < 0)
+     len = -len;
+   do
+    {
+      c1 = tolower(*p1++);
+      c2 = tolower(*p2++);
+      if (   !c1
+          || c1 != c2)
+        return(c1-c2);
+    } while (--len);
+
+   return(c1-c2);
+}
+#endif /* !HAVE_STRNCASECMP */
+
+
+
    LOCAL int
 days_of_february (year)
    const int year;
@@ -333,7 +436,7 @@ days_of_february (year)
    Computes the number of days in February and returns them.
 */
 {
-   return (year&3) ? 28 : (!(year%100)&&(year%400)) ? 28 : 29;
+   return((year&3) ? 28 : (!(year%100)&&(year%400)) ? 28 : 29);
 }
 
 
@@ -354,9 +457,9 @@ valid_date (day, month, year)
            && (day > dvec[month-1]))
        || (   (month == 2)
            && (day > days_of_february (year))))
-     return FALSE;
+     return(FALSE);
 
-   return TRUE;
+   return(TRUE);
 }
 
 
@@ -367,7 +470,7 @@ prev_date (day, month, year)
    int *month;
    int *year;
 /*
-   Sets a delivered date back by one day (to yesterdays date).
+   Sets a delivered date back by one day (to yesterday's date).
 */
 {
    (*day)--;
@@ -395,7 +498,7 @@ next_date (day, month, year)
    int *month;
    int *year;
 /*
-   Sets the delivered date forwards by one day (to tomorrows date).
+   Sets the delivered date forwards by one day (to tomorrow's date).
 */
 {
    (*day)++;
@@ -450,13 +553,17 @@ main (argc, argv)
      otherwise it will be searched using the $PATH environment variable.
 */
 {
-   auto     Slint  shift=1L;
+   auto     Slint  shift=(Slint)CHR2DIG(*SHIFT_VALUE_DFT);
    register int    len_year_max;
    register int    i;
+   register int    act_len;
+   register int    arg_len;
+   register int    status;
    auto     int    day;
    auto     int    month;
    auto     int    year;
    auto     char  *gcal_name;
+   auto     char  *gcal_line;
    auto     char  *ptr_char;
    auto     char  *buf_ptr_char;
    auto     Bool   shift_set=FALSE;
@@ -511,7 +618,10 @@ main (argc, argv)
    if ((Uint)i >= maxlen_max)
      s1 = my_realloc ((VOID_PTR)s1, i+1, 124, __FILE__, (long)__LINE__, "s1", i+1);
    if (!i)
-     strcpy(s1, "tcal");
+    {
+      strcpy(s1, rcsid+5);
+      s1[5] = '\0';
+    }
    else
      strcpy(s1, argv[0]);
 #ifdef SUFFIX_SEP
@@ -553,6 +663,25 @@ main (argc, argv)
     }
    prgr_name = (char *)my_malloc (i+1, 124, __FILE__, (long)__LINE__, "prgr_name", 0);
    strcpy(prgr_name, ptr_char);
+#if HAVE_SIGNAL
+   /*
+      Now let's modify the signal handling a bit to make sure that
+        temporary files are always deleted if such signals are raised.
+   */
+#  ifdef SIGINT
+   if (signal(SIGINT, SIG_IGN) != SIG_IGN)
+     (void)signal(SIGINT, (Sig_type)handle_signal);
+#  endif
+#  ifdef SIGTERM
+   if (signal(SIGTERM, SIG_IGN) != SIG_IGN)
+     (void)signal(SIGTERM, (Sig_type)handle_signal);
+#  endif
+#  ifdef SIGHUP
+   if (signal(SIGHUP, SIG_IGN) != SIG_IGN)
+     (void)signal(SIGHUP, (Sig_type)handle_signal);
+#  endif
+#endif /* HAVE_SIGNAL */
+   buf_ptr_char = SHIFT_VALUE_DFT;
    if (argc > 1)
     {
       /*
@@ -565,11 +694,11 @@ main (argc, argv)
          auto Bool  is_number=TRUE;
 
 
-         if (!strncmp(argv[1]+2, "help", strlen(argv[1]+2)))
+         if (!strncasecmp(argv[1]+2, "help", strlen(argv[1]+2)))
            usage_msg (stdout, prgr_name, 0);
-         if (!strncmp(argv[1]+2, "version", strlen(argv[1]+2)))
-           version_msg (stdout, 0);
-         if (!strncmp(argv[1]+2, "shi", 3))
+         if (!strncasecmp(argv[1]+2, "version", strlen(argv[1]+2)))
+           version_msg (stdout, prgr_name, 0);
+         if (!strncasecmp(argv[1]+2, "shi", 3))
           {
             ptr_char = strchr(argv[1]+2, *LARG_SEP);
             if (   ptr_char == (char *)NULL
@@ -579,13 +708,13 @@ main (argc, argv)
                /*
                   Error, option requires an argument.
                */
-#if USE_GER
-               fprintf(stderr, "%s: Option `%s' ben"OE"tigt ein Argument",
-                       prgr_name, argv[1]);
-#else /* !USE_GER */
+#if USE_DE
+               fprintf(stderr, "%s: Option `%s' ben%stigt ein Argument",
+                       prgr_name, argv[1], OE);
+#else /* !USE_DE */
                fprintf(stderr, _("%s: option `%s' requires an argument"),
                        prgr_name, argv[1]);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
                S_NEWLINE(stderr);
                usage_msg (stderr, prgr_name, 1);
              }
@@ -597,19 +726,19 @@ main (argc, argv)
                   case 3:
                   case 4:
                   case 5:
-                    if (!strncmp(argv[1]+2, "shift", i))
+                    if (!strncasecmp(argv[1]+2, "shift", i))
                       break;
                   default:
                     /*
                        Error, unrecognized option.
                     */
-#if USE_GER
+#if USE_DE
                     fprintf(stderr, "%s: unbekannte Option `%s'",
                             prgr_name, argv[1]);
-#else /* !USE_GER */
+#else /* !USE_DE */
                     fprintf(stderr, _("%s: unrecognized option `%s'"),
                             prgr_name, argv[1]);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
                     S_NEWLINE(stderr);
                     usage_msg (stderr, prgr_name, 1);
                 }
@@ -639,13 +768,13 @@ main (argc, argv)
                   /*
                      Error, invalid argument.
                   */
-#if USE_GER
-                  fprintf(stderr, "%s: Option mit unzul"AE"ssigem Argument -- %s",
-                          prgr_name, argv[1]);
-#else /* !USE_GER */
+#if USE_DE
+                  fprintf(stderr, "%s: Option mit unzul%sssigem Argument -- %s",
+                          prgr_name, AE, argv[1]);
+#else /* !USE_DE */
                   fprintf(stderr, _("%s: option with invalid argument -- %s"),
                           prgr_name, argv[1]);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
                   S_NEWLINE(stderr);
                   usage_msg (stderr, prgr_name, 1);
                 } 
@@ -674,81 +803,72 @@ main (argc, argv)
    */
    if (shift > 0L)
      for ( ; shift ; shift--)
-       next_date (&day, &month, &year);
+      {
+        next_date (&day, &month, &year);
+        if (year > YEAR_MAX)
+          my_error (2, "", 0L, buf_ptr_char, 0);
+      }
    else
      if (shift < 0L)
        for ( ; shift ; shift++)
-         prev_date (&day, &month, &year);
-   if (   (year >= YEAR_MIN)
-       && (year <= YEAR_MAX))
-    {
-      register int    act_len;
-      register int    arg_len;
-      register int    status;
-      auto     char  *gcal_line=(char *)my_malloc (MAXLEN_MAX, 124, __FILE__, (long)__LINE__, "gcal_line", 0);
-
-
-      /*
-         Create the base command line including the actual date modifier %...
-           which is used in the system() call to run Gcal.
-      */
-      sprintf(gcal_line, "%s %c%0*d%02d%02d",
-              gcal_name, RC_ADATE_CHAR, len_year_max, year, month, day);
-      act_len = (int)strlen(gcal_line);
-      /*
-         Skip the locally respected `--shift=[+|-]NUMBER' long-style option
-           if given in the command line.
-      */
-      if (shift_set)
-       {
-         argc--;
-         argv++;
-       }
-      /*
-         Now append all command line arguments into "gcal_line".
-      */
-      while (argc > 1)
-       {
-         strcat(gcal_line, " ");
-         argv++;
-         arg_len = (int)strlen(*argv);
-         if ((Uint)act_len+arg_len >= maxlen_max)
-          {
-            maxlen_max <<= 1;
-            if (maxlen_max > testval)
-              maxlen_max = testval;
-            gcal_line = (char *)my_realloc ((VOID_PTR)gcal_line, maxlen_max,
-                                            124, __FILE__, (long)__LINE__,
-                                            "gcal_line", maxlen_max);
-          }
-         strcat(gcal_line, *argv);
-         act_len += (arg_len + 1);
-         argc--;
-       }
-      status = system(gcal_line);
-      if (status == -1)
-       {
-#if USE_GER
-         fprintf(stderr, "%s: Fehler bei Programmausf"UE"hrung von `%s'\n",
-                 prgr_name, gcal_name);
-#else /* !USE_GER */
-         fprintf(stderr, _("%s: error during program execution of `%s'\n"),
-                 prgr_name, gcal_name);
-#endif /* !USE_GER */
-#if HAVE_ERRNO_H
-         perror(gcal_name);
-         exit(status);
-#else /* !HAVE_ERRNO_H */
-         exit(1);
-#endif /* !HAVE_ERRNO_H */
-       }
-
-      return status;
-    }
-   my_error (2, "", 0L, buf_ptr_char, 0);
-
+        {
+          prev_date (&day, &month, &year);
+          if (year < YEAR_MIN)
+            my_error (2, "", 0L, buf_ptr_char, 0);
+        }
    /*
-      This makes the compiler happy!
+      Create the base command line including the actual date modifier %...
+        which is used in the system() call to run Gcal.
    */
-   return 0;
+   gcal_line = (char *)my_malloc (MAXLEN_MAX, 124, __FILE__, (long)__LINE__, "gcal_line", 0);
+   sprintf(gcal_line, "%s %c%0*d%02d%02d",
+           gcal_name, RC_ADATE_CHAR, len_year_max, year, month, day);
+   act_len = (int)strlen(gcal_line);
+   /*
+      Skip the locally respected `--shift=[+|-]NUMBER' long-style option
+        if given in the command line.
+   */
+   if (shift_set)
+    {
+      argc--;
+      argv++;
+    }
+   /*
+      Now append all command line arguments to "gcal_line".
+   */
+   while (argc > 1)
+    {
+      strcat(gcal_line, " ");
+      argv++;
+      arg_len = (int)strlen(*argv);
+      if ((Uint)act_len+arg_len >= maxlen_max)
+       {
+         maxlen_max <<= 1;
+         if (maxlen_max > testval)
+           maxlen_max = testval;
+         gcal_line = (char *)my_realloc ((VOID_PTR)gcal_line, maxlen_max,
+                                         124, __FILE__, (long)__LINE__,
+                                         "gcal_line", maxlen_max);
+       }
+      strcat(gcal_line, *argv);
+      act_len += (arg_len + 1);
+      argc--;
+    }
+   status = system(gcal_line);
+   if (status == -1)
+    {
+#if USE_DE
+      fprintf(stderr, "%s: Fehler bei Programmausf%shrung von `%s'\n",
+              prgr_name, UE, gcal_name);
+#else /* !USE_DE */
+      fprintf(stderr, _("%s: error during program execution of `%s'\n"),
+              prgr_name, gcal_name);
+#endif /* !USE_DE */
+#if HAVE_ERRNO_H
+      perror(gcal_name);
+#endif
+      exit(status);
+    }
+
+   return(status);
 }
