@@ -2,7 +2,7 @@
 *  txt2gcal.c:  Creates a verbatim Gcal resource file from a text file.
 *
 *
-*  Copyright (C) 1996 Thomas Esken
+*  Copyright (C) 1996, 1997 Thomas Esken
 *
 *  This software doesn't claim completeness, correctness or usability.
 *  On principle I will not be liable for ANY damages or losses (implicit
@@ -25,21 +25,21 @@
 
 
 
-static char rcsid[]="$Id: txt2gcal.c 2.40 1996/07/16 02:04:00 tom Exp $";
+static char rcsid[]="$Id: txt2gcal.c 2.94 1997/03/31 02:09:04 tom Exp $";
 
 
 
-#include "gcal_tai.h"
+#include "tailor.h"
 #if HAVE_CTYPE_H
 #  include <ctype.h>
 #endif
 #if HAVE_ERRNO_H
 #  include <errno.h>
 #endif
+#if (!HAVE_SIGNAL_H || !HAVE_SIGNAL) && HAVE_SYS_TYPES_H   /* Otherwise "gcal.h" includes <sys/types.h> */
+#  include <sys/types.h>
+#endif
 #if HAVE_SYS_STAT_H
-#  if HAVE_SYS_TYPES_H
-#    include <sys/types.h>
-#  endif
 #  include <sys/stat.h>
 #endif
 #ifndef USE_RC
@@ -57,12 +57,13 @@ extern "C"
 {
 #endif
 LOCAL void
-usage_msg __P_((FILE *fp,
-                char *prgr_name,
-                int   exit_status));
+usage_msg __P_((      FILE *fp,
+                const char *prgr_name,
+                      int   exit_status));
 LOCAL void
-version_msg __P_((FILE *fp,
-                  int   exit_status));
+version_msg __P_((      FILE *fp,
+                  const char *prgr_name,
+                        int   exit_status));
 LOCAL VOID_PTR
 my_malloc __P_((const int   amount,
                 const int   exit_status,
@@ -84,6 +85,16 @@ my_error __P_((const int   exit_status,
                const long  module_line,
                const char *var_name,
                const int   var_contents));
+#if HAVE_SIGNAL && (defined(SIGINT) || defined(SIGTERM) || defined(SIGHUP))
+LOCAL RETSIGTYPE
+handle_signal __P_((int the_signal));
+#endif
+#if !HAVE_STRNCASECMP
+LOCAL int
+my_strncasecmp __P_((const char *s1,
+                     const char *s2,
+                           int   len));
+#endif /* !HAVE_STRNCASECMP */
 EXPORT int
 main __P_((int   argc,
            char *argv[]));
@@ -108,38 +119,70 @@ LOCAL char  *prgr_name;               /* The name of this executable */
 
    LOCAL void
 usage_msg (fp, prgr_name, exit_status)
-   FILE *fp;
-   char *prgr_name;
-   int   exit_status;
+         FILE *fp;
+   const char *prgr_name;
+         int   exit_status;
 /*
    Writes the program "usage" text to file `fp' and
      terminates the program with `exit_status'.
 */
 {
-#if USE_GER
+#if USE_DE
    fprintf(fp, "Aufruf:  %s  [--help | --version] | [TEXTDATEI | -] [DATUMTEIL]\n", prgr_name);
-#else /* !USE_GER */
+   if (!exit_status)
+    {
+      S_NEWLINE(fp);
+      fprintf(fp, "Fehlerberichte via eMail an <esken@uni-muenster.de>");
+      S_NEWLINE(fp);
+      fprintf(fp, "oder (falls das fehlschl%sgt) an <bug-gnu-utils@prep.ai.mit.edu>.", AE);
+      S_NEWLINE(fp);
+    }
+#else /* !USE_DE */
    fprintf(fp, _("Usage:  %s  [--help | --version] | [TEXT-FILE | -] [DATE-PART]\n"), prgr_name);
-#endif /* !USE_GER */
+   if (!exit_status)
+    {
+      S_NEWLINE(fp);
+      fprintf(fp, _("Email bug reports to <esken@uni-muenster.de>"));
+      S_NEWLINE(fp);
+      fprintf(fp, _("or (if this fails) to <bug-gnu-utils@prep.ai.mit.edu>."));
+      S_NEWLINE(fp);
+    }
+#endif /* !USE_DE */
    exit(exit_status);
 }
 
 
 
    LOCAL void
-version_msg (fp, exit_status)
-   FILE *fp;
-   int   exit_status;
+version_msg (fp, prgr_name, exit_status)
+         FILE *fp;
+   const char *prgr_name;
+         int   exit_status;
 /*
    Writes the program "version" text to file `fp' and
      terminates the program with `exit_status'.
 */
 {
-   char *ptr_rcsid=rcsid+5;
+   auto char *ptr_rcsid=rcsid+16;
 
 
-   rcsid[40] = '\0';
-   fprintf(fp, "%s\n", ptr_rcsid);
+   rcsid[20] = '\0';
+   fprintf(fp, "%s (GNU cal %s) %s\n", prgr_name, VERSION_NO, ptr_rcsid);
+   fprintf(fp, "Copyright (C) 1996-1997 Thomas Esken\n");
+#if USE_DE
+   fprintf(fp, "Dies ist freie Software; in den Quellen befindet sich die Lizenz-");
+   S_NEWLINE(fp);
+   fprintf(fp, "und Kopierbedingung.  Es gibt KEINERLEI Garantie, nicht einmal f%sr", UE);
+   S_NEWLINE(fp);
+   fprintf(fp, "die TAUGLICHKEIT oder die VERWENDBARKEIT ZU EINEM ANGEGEBENEN ZWECK.");
+#else /* !USE_DE */
+   fprintf(fp, _("This is free software; see the source for copying conditions."));
+   S_NEWLINE(fp);
+   fprintf(fp, _("There is NO warranty, without even the implied warranty of"));
+   S_NEWLINE(fp);
+   fprintf(fp, _("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."));
+#endif /* !USE_DE */
+   S_NEWLINE(fp);
    exit(exit_status);
 }
 
@@ -174,7 +217,7 @@ my_malloc (amount, exit_status, module_name, module_line, var_name, var_contents
      */
      my_error (exit_status, module_name, module_line, var_name, var_contents);
 
-   return ptr_memblock;
+   return(ptr_memblock);
 }
 
 
@@ -201,7 +244,7 @@ my_realloc (ptr_memblock, amount, exit_status, module_name, module_line, var_nam
      */
      my_error (107, module_name, module_line, var_name, (int)testval);
    if (ptr_memblock == (VOID_PTR)NULL)
-     return my_malloc (amount, exit_status, module_name, module_line, var_name, var_contents);
+     return(my_malloc (amount, exit_status, module_name, module_line, var_name, var_contents));
    ptr_memblock = (VOID_PTR)realloc(ptr_memblock, (int)amount);
    if (ptr_memblock == (VOID_PTR)NULL)
      /*
@@ -209,7 +252,7 @@ my_realloc (ptr_memblock, amount, exit_status, module_name, module_line, var_nam
      */
      my_error (exit_status, module_name, module_line, var_name, var_contents);
 
-   return ptr_memblock;
+   return(ptr_memblock);
 }
 
 
@@ -226,26 +269,26 @@ my_error (exit_status, module_name, module_line, var_name, var_contents)
      and terminates the program with status `exit_status'.
 */
 {
-#if USE_GER
+#if USE_DE
    fprintf(stderr, "\n%s: Abbruch, ", prgr_name);
-#else /* !USE_GER */
+#else /* !USE_DE */
    fprintf(stderr, _("\n%s: abort, "), prgr_name);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
    switch(exit_status)
     {
-#if USE_GER
+#if USE_DE
       case 124:
-        fprintf(stderr, "`%s' Zeile %ld: virtueller Speicher ersch"OE"pft (%s=%d)",
-                module_name, module_line, var_name, var_contents);
+        fprintf(stderr, "`%s' Zeile %ld: virtueller Speicher ersch%spft (%s=%d)",
+                module_name, module_line, OE, var_name, var_contents);
         break;
       case 107:
-        fprintf(stderr, "`%s' Zeile %ld: (`%s') ung"UE"ltiger Wert f"UE"r Tabellengr"OE""SZ"e `sizeof %s>%d'",
-                module_name, module_line, INTERNAL_TXT, var_name, var_contents);
+        fprintf(stderr, "`%s' Zeile %ld: (`%s') ung%sltiger Wert f%sr Tabellengr%s%se `sizeof %s>%d'",
+                module_name, module_line, INTERNAL_TXT, UE, UE, OE, SZ, var_name, var_contents);
       default:
         fprintf(stderr, "`%s' Zeile %ld: (`%s') unbehandelter Fehler (%d)",
                 module_name, module_line, INTERNAL_TXT, exit_status);
         break;
-#else /* !USE_GER */
+#else /* !USE_DE */
       case 124:
         fprintf(stderr, _("`%s' line %ld: virtual memory exhausted (%s=%d)"),
                 module_name, module_line, var_name, var_contents);
@@ -257,7 +300,7 @@ my_error (exit_status, module_name, module_line, var_name, var_contents)
       default:
         fprintf(stderr, _("`%s' line %ld: (`%s') unmanaged error (%d)"),
                 module_name, module_line, _("Internal"), exit_status);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
     }
    S_NEWLINE(stderr);
    exit(exit_status);
@@ -265,6 +308,63 @@ my_error (exit_status, module_name, module_line, var_name, var_contents)
 
 
 
+#if HAVE_SIGNAL && (defined(SIGINT) || defined(SIGTERM) || defined(SIGHUP))
+   LOCAL RETSIGTYPE
+handle_signal (the_signal)
+   int the_signal;
+/*
+   Signal handler function which displays the numeric ID of the received signal
+     on stderr channel and terminates the program with an exit status 3.
+*/
+{
+   fflush(stdout);
+#if USE_DE
+   fprintf(stderr, "\n%s: Programmabbruch durch Signal %d\n", prgr_name, the_signal);
+#else /* !USE_DE */
+   fprintf(stderr, _("\n%s: program aborted by signal %d\n"), prgr_name, the_signal);
+#endif /* */  
+   exit(3);
+}
+#endif /* HAVE_SIGNAL && (SIGINT || SIGTERM || SIGHUP) */
+
+
+
+#if !HAVE_STRNCASECMP
+   PUBLIC int
+my_strncasecmp (s1, s2, len)
+   const char *s1;
+   const char *s2;
+         int   len;
+/*
+   Same as the ANSI C `strncmp()' function, but case insensitive.
+*/
+{
+   auto const Uchar  *p1=(const Uchar *)s1;
+   auto const Uchar  *p2=(const Uchar *)s2;
+   auto       Uchar   c1;
+   auto       Uchar   c2;
+
+
+   if (   !len
+       || p1 == p2)
+     return(0);
+   if (len < 0)
+     len = -len;
+   do
+    {
+      c1 = tolower(*p1++);
+      c2 = tolower(*p2++);
+      if (   !c1
+          || c1 != c2)
+        return(c1-c2);
+    } while (--len);
+
+   return(c1-c2);
+}
+#endif /* !HAVE_STRNCASECMP */
+
+
+
    PUBLIC int
 main (argc, argv)
    int   argc;
@@ -273,7 +373,7 @@ main (argc, argv)
    Creates a verbatim Gcal resource file from a given text file
      (optional argument 1) with a given date-part (optional argument 2) and
      displays the results on the stdout channel.  If no text file name
-     or only a dash `-' is given, read input from stdin channel.
+     or only a dash '-' is given, read input from stdin channel.
 */
 {
    auto     FILE  *fp=(FILE *)NULL;
@@ -331,7 +431,10 @@ main (argc, argv)
    if ((Uint)i >= maxlen_max)
      s1 = my_realloc ((VOID_PTR)s1, i+1, 124, __FILE__, (long)__LINE__, "s1", i+1);
    if (!i)
-     strcpy(s1, "txt2gcal");
+    {
+      strcpy(s1, rcsid+5);
+      s1[9] = '\0';
+    }
    else
      strcpy(s1, argv[0]);
 #ifdef SUFFIX_SEP
@@ -375,6 +478,24 @@ main (argc, argv)
     }
    prgr_name = (char *)my_malloc (i+1, 124, __FILE__, (long)__LINE__, "prgr_name", 0);
    strcpy(prgr_name, ptr_char);
+#if HAVE_SIGNAL
+   /*
+      Now let's modify the signal handling a bit to make sure that
+        temporary files are always deleted if such signals are raised.
+   */
+#  ifdef SIGINT
+   if (signal(SIGINT, SIG_IGN) != SIG_IGN)
+     (void)signal(SIGINT, (Sig_type)handle_signal);
+#  endif
+#  ifdef SIGTERM
+   if (signal(SIGTERM, SIG_IGN) != SIG_IGN)
+     (void)signal(SIGTERM, (Sig_type)handle_signal);
+#  endif
+#  ifdef SIGHUP
+   if (signal(SIGHUP, SIG_IGN) != SIG_IGN)
+     (void)signal(SIGHUP, (Sig_type)handle_signal);
+#  endif
+#endif /* HAVE_SIGNAL */
    if (argc > 1)
     {
       if (   (strlen(argv[1]) == 1)
@@ -389,18 +510,18 @@ main (argc, argv)
              && (*argv[1] == *SWITCH)
              && (*(argv[1]+1) == *SWITCH))
           {
-            if (!strncmp(argv[1]+2, "help", strlen(argv[1]+2)))
+            if (!strncasecmp(argv[1]+2, "help", strlen(argv[1]+2)))
               usage_msg (stdout, prgr_name, 0);
-            if (!strncmp(argv[1]+2, "version", strlen(argv[1]+2)))
-              version_msg (stdout, 0);
+            if (!strncasecmp(argv[1]+2, "version", strlen(argv[1]+2)))
+              version_msg (stdout, prgr_name, 0);
             /*
                Error, unknown long-style option given.  
             */
-#if USE_GER
+#if USE_DE
             fprintf(stderr, "%s: unbekannte Option `%s'", prgr_name, argv[1]);
-#else /* !USE_GER */
+#else /* !USE_DE */
             fprintf(stderr, _("%s: unrecognized option `%s'"), prgr_name, argv[1]);
-#endif /* !USE_GER */
+#endif /* !USE_DE */
             S_NEWLINE(stderr);
             usage_msg (stderr, prgr_name, 1);
           }
@@ -437,19 +558,19 @@ main (argc, argv)
       sprintf(s1, "%s: (%s) ", prgr_name, argv[1]);
 #if HAVE_ERRNO_H
       if (!is_regular_file)
-#  if USE_GER
-        fprintf(stderr, "%s: keine regulaere Datei\n", s1);
-#  else /* !USE_GER */
+#  if USE_DE
+        fprintf(stderr, "%s: keine regul%sre Datei\n", s1, AE);
+#  else /* !USE_DE */
         fprintf(stderr, _("%s: no regular file\n"), s1);
-#  endif /* !USE_GER */
+#  endif /* !USE_DE */
       else
         perror(s1);
 #else /* !HAVE_ERRNO_H */
-#  if USE_GER
+#  if USE_DE
       fprintf(stderr, "%s: Datei nicht gefunden\n", s1);
-#  else /* !USE_GER */
+#  else /* !USE_DE */
       fprintf(stderr, _("%s: file not found\n"), s1);
-#  endif /* !USE_GER */
+#  endif /* !USE_DE */
 #endif /* !HAVE_ERRNO_H */
       exit(1);
     }
@@ -470,15 +591,16 @@ main (argc, argv)
                   fprintf(stdout, "%c %c%c%c", ch, RC_NL_CHAR, ch, ch2);
                   break;
                 case RC_NL_CHAR:
+                case RC_NL2_CHAR:
                 case RC_TVAR_CHAR:
-                case RC_MACRO_CHAR:
+                case RC_SPECIAL_TEXT_CHAR:
                   fprintf(stdout, "%c%c%c", ch, ch, ch2);
                   break;
                 default:
                   fprintf(stdout, "%c%c", ch, ch2);
               }
            else
-             fprintf(stdout, "%c", ch);
+             fputc(ch, stdout);
            break;
          case RC_TVAR_CHAR:
            ch2 = fgetc(fp);
@@ -490,6 +612,7 @@ main (argc, argv)
               switch (ch2)
                {
                  case RC_NL_CHAR:
+                 case RC_NL2_CHAR:
                    fprintf(stdout, "%c%c%c", ch, QUOTE_CHAR, ch2);
                    break;
                  case '\n':
@@ -500,29 +623,94 @@ main (argc, argv)
                }
             }
            else
-             fprintf(stdout, "%c", ch);
+             fputc(ch, stdout);
            break;
-         case RC_MACRO_CHAR:
+         case RC_SPECIAL_TEXT_CHAR:
            ch2 = fgetc(fp);
            if (ch2 != EOF)
-             switch (tolower(ch2))
+             switch (ch2)
               {
-                case RC_SDATE_CHAR:
+                case RC_SHELL_ESC_CHAR:
+                case RC_EX_LHDY_CHAR:
+                case RC_EX_NLHDY_CHAR:
+                case RC_EX_AHDY_CHAR:
+                case RC_EX_NAHDY_CHAR:
+                case RC_EX_MON_CHAR:
+                case RC_EX_NMON_CHAR:
+                case RC_EX_TUE_CHAR:
+                case RC_EX_NTUE_CHAR:
+                case RC_EX_WED_CHAR:
+                case RC_EX_NWED_CHAR:
+                case RC_EX_THU_CHAR:
+                case RC_EX_NTHU_CHAR:
+                case RC_EX_FRI_CHAR:
+                case RC_EX_NFRI_CHAR:
+                case RC_EX_SAT_CHAR:
+                case RC_EX_NSAT_CHAR:
+                case RC_EX_SUN_CHAR:
+                case RC_EX_NSUN_CHAR:
+                case RC_EX_MON_2_THU_CHAR:
+                case RC_EX_NMON_2_THU_CHAR:
+                case RC_EX_MON_2_FRI_CHAR:
+                case RC_EX_NMON_2_FRI_CHAR:
+                case RC_IDATE_CHAR:
                 case RC_EDATE_CHAR:
-                case RC_WNAME_CHAR:
-                case RC_WNUMBER_CHAR:
+                case RC_WDNAME_CHAR:
+                case RC_WDNAME3_CHAR:
+                case RC_WDNAME2_CHAR:
+                case RC_WDNR_M1_2_S7_CHAR:
+                case RC_WDNR_M0_2_S6_CHAR:
+                case RC_WDNR_M1_2_S7S_CHAR:
+                case RC_WDNR_M0_2_S6S_CHAR:
+                case RC_WDNR_S1_2_S7_CHAR:
+                case RC_WDNR_S0_2_S6_CHAR:
+                case RC_WDNR_S1_2_S7S_CHAR:
+                case RC_WDNR_S0_2_S6S_CHAR:
+                case RC_WDNR_X1_2_X7_CHAR:
+                case RC_WDNR_X0_2_X6_CHAR:
+                case RC_WDNR_X1_2_X7S_CHAR:
+                case RC_WDNR_X0_2_X6S_CHAR:
+                case RC_DOYNR_NZ_CHAR:
+                case RC_DOYNR_LZ_CHAR:
+                case RC_DOYNR_NZS_CHAR:
+                case RC_DOYNR_LZS_CHAR:
+                case RC_DAYNR_NZ_CHAR:
+                case RC_DAYNR_LZ_CHAR:
+                case RC_DAYNR_NZS_CHAR:
+                case RC_DAYNR_LZS_CHAR:
+                case RC_MONTHNAME_CHAR:
+                case RC_MONTHNAME3_CHAR:
+                case RC_MONTHNR_NZ_CHAR:
+                case RC_MONTHNR_LZ_CHAR:
+                case RC_MONTHNR_NZS_CHAR:
+                case RC_MONTHNR_LZS_CHAR:
+                case RC_YEARNR_NZ_CHAR:
+                case RC_YEARNR_LZ_CHAR:
+                case RC_WEEKNR_CHAR:
                 case RC_BYEAR_CHAR:
-                case RC_DATE_CHAR:
-                case RC_DAY_CHAR:
-                case RC_WEEK_CHAR:
-                case RC_MONTH_CHAR:
-                case RC_YEAR_CHAR:
-                case RC_TIME_CHAR:
+                case RC_BYEAR_S_CHAR:
+                case RC_MOON_NZ_CHAR:
+                case RC_MOON_LZ_CHAR:
+                case RC_MOON_IMAGE_CHAR:
+                case RC_BIO_CHAR:
+                case RC_BIO_BAR_CHAR:
+                case RC_TDATE_CHAR:
                 case RC_JDAYS_CHAR:
+                case RC_TIME_CHAR:
+                case RC_DAY_DIFF_CHAR:
+                case RC_WEEK_DIFF_CHAR:
+                case RC_MONTH_DIFF_CHAR:
+                case RC_YEAR_DIFF_CHAR:
                 case RC_HLS1S_CHAR:
                 case RC_HLS1E_CHAR:
                 case RC_HLS2S_CHAR:
                 case RC_HLS2E_CHAR:
+                case RC_HLS3S_CHAR:
+                case RC_HLS3E_CHAR:
+                case RC_HLS4S_CHAR:
+                case RC_HLS4E_CHAR:
+                case RC_HLS5S_CHAR:
+                case RC_HLS5E_CHAR:
                   fputc(QUOTE_CHAR, stdout);
                   ch3 = '\0';
                 default:
@@ -531,8 +719,13 @@ main (argc, argv)
                   switch (ch2)
                    {
                      case RC_NL_CHAR:
+                     case RC_NL2_CHAR:
                        fprintf(stdout, "%c%c%c", ch, QUOTE_CHAR, ch2);
                        break;
+                     case RC_SPECIAL_TEXT_CHAR:
+                       fputc(ch, stdout);
+                       ungetc(ch2, fp);
+                       break;
                      case '\n':
                        fprintf(stdout, "%c%c%c%c", ch, RC_NL_CHAR, QUOTE_CHAR, ch2);
                        break;
@@ -541,9 +734,10 @@ main (argc, argv)
                    }
               }
            else
-             fprintf(stdout, "%c", ch);
+             fputc(ch, stdout);
            break;
          case RC_NL_CHAR:
+         case RC_NL2_CHAR:
            fputc(QUOTE_CHAR, stdout);
          default:
            switch (ch)
@@ -566,14 +760,14 @@ main (argc, argv)
               default:
                 if (   isspace(ch)
                     && !is_trailing_whitespace)
-                  fprintf(stdout, "%c", QUOTE_CHAR);
+                  fputc(QUOTE_CHAR, stdout);
                 is_trailing_whitespace = TRUE;
-                fprintf(stdout, "%c", ch);
+                fputc(ch, stdout);
             }
        }
       ch3 = ch2;
     }
    fclose(fp);
 
-   return 0;
+   return(0);
 }
